#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Josh Montague"
__license__="MIT License"

import sys
import unittest
from StringIO import StringIO
from tumblr_acs import *


# valid activity from source (eg from data/) 
VALID_ACTIVITY = """{"id":"tag:gnip.tumblr.com:2012:post/45837266698/post","tumblrRebloggedRoot":{"author":{"link":"http://the-personal-quotes.tumblr.com/","displayName":"the personal quotes"},"link":"http://the-personal-quotes.tumblr.com/post/18729698371"},"verb":"post","target":{"link":"http://stardustgrass.tumblr.com/","displayName":"stardustgrass","objectType":"blog"},"postedTime":"2013-03-20T15:09:40+00:00","actor":{"id":"tag:gnip.tumblr.com:2012:blog/76962586"},"object":{"tags":[{"displayName":"the help"},{"displayName":"gif"}],"summary":"","tumblrNoteCount":303949,"objectTypes":["image"],"link":"http://stardustgrass.tumblr.com/post/45837266698","postedTime":"2013-03-20T15:09:40+00:00","tumblrFormat":"html","id":"tag:gnip.tumblr.com:2012:post/45837266698","source":{"link":"http://the-personal-quotes.tumblr.com/post/18729698371","displayName":"the-personal-quotes"},"items":[{"summary":"","image":{"height":52,"width":100,"link":"http://25.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_100.gif"},"objectType":"image","fullImage":{"height":260,"width":500,"link":"http://24.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_500.gif"}}],"tumblrReblogKey":"57dc6dOl","totalItems":1,"tumblrType":"photo","objectType":"photo-album"},"tumblrRebloggedFrom":{"author":{"link":"http://illllest.tumblr.com/","displayName":"Unknown Pleasures"},"link":"http://illllest.tumblr.com/post/45832750088"},"gnip":{}}"""


class TestTumblrACS(unittest.TestCase):
    """Unit tests of common CSV utility functions"""

    def setUp(self):
        """
        TblracsCSV constructor takes  args - create one example of each for 
        use in all tests (loop over the self.objs list).
        """ 

        self.delim = '|'

        # use a dict to make it easier to refer to the particular object cases
        self.objs = { 
                    "base": TblracsCSV(self.delim, None, False, False, False, False)
                    , "user": TblracsCSV(self.delim, None, True, False, False, False)
                    , "rules": TblracsCSV(self.delim, None, False, True, False, False)
                    , "lang": TblracsCSV(self.delim, None, False, False, True, False)
                    , "struct": TblracsCSV(self.delim, None, False, False, False, True)
                    , "all": TblracsCSV(self.delim, None, True, True, True, True)
                    , "keypath": TblracsCSV(self.delim, "verb", False, False, False, False)
                    }

        # set any instance attributes here, avoid hard-coding in test methods 
        self.base_length = 4
        # count of extra fields added by each of these options
        self.keypath_length = 1
        self.user_length = 2
        self.rules_length = 1
        self.lang_length = 1
        self.struct_length = 3
        self.all_length = self.base_length \
                            + self.user_length \
                            + self.lang_length \
                            + self.struct_length \
                            + self.rules_length 

    def tearDown(self):
        """Nothing to do here."""
        pass

    def test_sample_data(self):
        """
        Check that we can use each test object's procRecord method on each record in the 
        tumblr_sample.json example file without raising an Exception.
        """
        # get a temporary object
        tmp = self.objs["base"]

        # grab the correct data file 
        datafile = "./data/tumblr_sample.json"

        # loop over all test tumblr processing objects
        for o in self.objs.values():
            # loop over records in test file 
            for i, record in o.file_reader(datafile):
                # if there's a problem parsing, this method will raise an Exception
                record_string = o.procRecord(record)


    def test_reader(self):
        """Test that our file_reader method is working as expected."""
        # grab the base instance 
        o = self.objs["base"]
        
        # without eg a for loop, we use the generator's next() method 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

   
    #
    # test the output (length) for each kind of TblracsCSV object 
    #
    
    def test_base_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the base instance 
        o = self.objs["base"]

        # use sample record in this module 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            # procRecord returns a delimited string
            record_string = o.procRecord(record)
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length 
                            )  


    def test_user_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["user"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.user_length 
                            ) 


    def test_struct_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["struct"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.struct_length 
                            )  


    def test_rules_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["rules"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.rules_length 
                            )  

    
    def test_lang_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["lang"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.lang_length 
                            )  

    
    def test_all_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["all"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            loop = True
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.all_length 
                            )  


    def test_keypath_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["keypath"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 1 extra field now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.keypath_length 
                            )  

    #
    # test that all of the fields we need to extract from the records actually work
    #

    def test_struct_fields(self):
        """
        Test for the presence of non-"None" values in a good record when using the 'struct' 
        option, and "None"s in an intentionally-damaged record.
        """ 

        # this activity (chosen from data/) has all the struct fields in it  
        GOOD_STRUCT_STRING = """{"id":"tag:gnip.tumblr.com:2012:post/45837266698/post","tumblrRebloggedRoot":{"author":{"link":"http://the-personal-quotes.tumblr.com/","displayName":"the personal quotes"},"link":"http://the-personal-quotes.tumblr.com/post/18729698371"},"verb":"post","target":{"link":"http://stardustgrass.tumblr.com/","displayName":"stardustgrass","objectType":"blog"},"postedTime":"2013-03-20T15:09:40+00:00","actor":{"id":"tag:gnip.tumblr.com:2012:blog/76962586"},"object":{"tags":[{"displayName":"the help"},{"displayName":"gif"}],"summary":"","tumblrNoteCount":303949,"objectTypes":["image"],"link":"http://stardustgrass.tumblr.com/post/45837266698","postedTime":"2013-03-20T15:09:40+00:00","tumblrFormat":"html","id":"tag:gnip.tumblr.com:2012:post/45837266698","source":{"link":"http://the-personal-quotes.tumblr.com/post/18729698371","displayName":"the-personal-quotes"},"items":[{"summary":"","image":{"height":52,"width":100,"link":"http://25.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_100.gif"},"objectType":"image","fullImage":{"height":260,"width":500,"link":"http://24.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_500.gif"}}],"tumblrReblogKey":"57dc6dOl","totalItems":1,"tumblrType":"photo","objectType":"photo-album"},"tumblrRebloggedFrom":{"author":{"link":"http://illllest.tumblr.com/","displayName":"Unknown Pleasures"},"link":"http://illllest.tumblr.com/post/45832750088"},"gnip":{}}"""

        # break the "tumblrRebloggedFrom" key in this activity, so that options_struct fields should all be "None" 
        BAD_STRUCT_STRING = """{"id":"tag:gnip.tumblr.com:2012:post/45837266698/post","tumblrRebloggedRoot":{"author":{"link":"http://the-personal-quotes.tumblr.com/","displayName":"the personal quotes"},"link":"http://the-personal-quotes.tumblr.com/post/18729698371"},"verb":"post","target":{"link":"http://stardustgrass.tumblr.com/","displayName":"stardustgrass","objectType":"blog"},"postedTime":"2013-03-20T15:09:40+00:00","actor":{"id":"tag:gnip.tumblr.com:2012:blog/76962586"},"object":{"tags":[{"displayName":"the help"},{"displayName":"gif"}],"summary":"","tumblrNoteCount":303949,"objectTypes":["image"],"link":"http://stardustgrass.tumblr.com/post/45837266698","postedTime":"2013-03-20T15:09:40+00:00","tumblrFormat":"html","id":"tag:gnip.tumblr.com:2012:post/45837266698","source":{"link":"http://the-personal-quotes.tumblr.com/post/18729698371","displayName":"the-personal-quotes"},"items":[{"summary":"","image":{"height":52,"width":100,"link":"http://25.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_100.gif"},"objectType":"image","fullImage":{"height":260,"width":500,"link":"http://24.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_500.gif"}}],"tumblrReblogKey":"57dc6dOl","totalItems":1,"tumblrType":"photo","objectType":"photo-album"},"tumblrRebloggedFromBAD":{"author":{"link":"http://illllest.tumblr.com/","displayName":"Unknown Pleasures"},"link":"http://illllest.tumblr.com/post/45832750088"},"gnip":{}}"""

        o = self.objs["struct"]

        # this should have data 
        for i, record in o.file_reader( json_string=GOOD_STRUCT_STRING ):
            record_string = o.procRecord(record)
            [ self.assertNotEqual( "None", x ) 
                for x in record_string.split(self.delim)[-self.struct_length:] 
            ] 
        
        # this should not have data in the 2nd and 3rd struct fields (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_STRUCT_STRING ):
            record_string = o.procRecord(record)
            [ self.assertEqual( "None", x ) 
                for x in record_string.split(self.delim)[-self.struct_length+1:] 
            ] 

    def test_summary_content_fields(self):
        """
        Test for the presence of non-"None" values in a good record,
        and "None"s in an intentionally-damaged record.
        """ 

        # this record contains an valid "object:summary" key path
        GOOD_CONTENT_SUMMARY_STRING = """{"id":"tag:gnip.tumblr.com:2012:post/45837266698/post","tumblrRebloggedRoot":{"author":{"link":"http://the-personal-quotes.tumblr.com/","displayName":"the personal quotes"},"link":"http://the-personal-quotes.tumblr.com/post/18729698371"},"verb":"post","target":{"link":"http://stardustgrass.tumblr.com/","displayName":"stardustgrass","objectType":"blog"},"postedTime":"2013-03-20T15:09:40+00:00","actor":{"id":"tag:gnip.tumblr.com:2012:blog/76962586"},"object":{"tags":[{"displayName":"the help"},{"displayName":"gif"}],"summary":"","tumblrNoteCount":303949,"objectTypes":["image"],"link":"http://stardustgrass.tumblr.com/post/45837266698","postedTime":"2013-03-20T15:09:40+00:00","tumblrFormat":"html","id":"tag:gnip.tumblr.com:2012:post/45837266698","source":{"link":"http://the-personal-quotes.tumblr.com/post/18729698371","displayName":"the-personal-quotes"},"items":[{"summary":"","image":{"height":52,"width":100,"link":"http://25.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_100.gif"},"objectType":"image","fullImage":{"height":260,"width":500,"link":"http://24.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_500.gif"}}],"tumblrReblogKey":"57dc6dOl","totalItems":1,"tumblrType":"photo","objectType":"photo-album"},"tumblrRebloggedFrom":{"author":{"link":"http://illllest.tumblr.com/","displayName":"Unknown Pleasures"},"link":"http://illllest.tumblr.com/post/45832750088"},"gnip":{}}"""

# neither "summary" nor "content" keys are in this record; should return None
        BAD_CONTENT_SUMMARY_STRING = """{"id":"tag:gnip.tumblr.com:2012:post/45837266698/post","tumblrRebloggedRoot":{"author":{"link":"http://the-personal-quotes.tumblr.com/","displayName":"the personal quotes"},"link":"http://the-personal-quotes.tumblr.com/post/18729698371"},"verb":"post","target":{"link":"http://stardustgrass.tumblr.com/","displayName":"stardustgrass","objectType":"blog"},"postedTime":"2013-03-20T15:09:40+00:00","actor":{"id":"tag:gnip.tumblr.com:2012:blog/76962586"},"object":{"tags":[{"displayName":"the help"},{"displayName":"gif"}],"tumblrNoteCount":303949,"objectTypes":["image"],"link":"http://stardustgrass.tumblr.com/post/45837266698","postedTime":"2013-03-20T15:09:40+00:00","tumblrFormat":"html","id":"tag:gnip.tumblr.com:2012:post/45837266698","source":{"link":"http://the-personal-quotes.tumblr.com/post/18729698371","displayName":"the-personal-quotes"},"items":[{"summary":"","image":{"height":52,"width":100,"link":"http://25.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_100.gif"},"objectType":"image","fullImage":{"height":260,"width":500,"link":"http://24.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_500.gif"}}],"tumblrReblogKey":"57dc6dOl","totalItems":1,"tumblrType":"photo","objectType":"photo-album"},"tumblrRebloggedFromBAD":{"author":{"link":"http://illllest.tumblr.com/","displayName":"Unknown Pleasures"},"link":"http://illllest.tumblr.com/post/45832750088"},"gnip":{}}"""

        o = self.objs["base"]

        # this should have data (an empty string)
        for i, record in o.file_reader( json_string=GOOD_CONTENT_SUMMARY_STRING ):
            record_string = o.procRecord(record)
            self.assertNotEqual( "None", record_string.split(self.delim)[2] )
        
        # this should return "None" because there is no "summary" key in the "object" field
        for i, record in o.file_reader( json_string=BAD_CONTENT_SUMMARY_STRING ):
            record_string = o.procRecord(record)
            #print(record_string)
            self.assertEqual( "None", record_string.split(self.delim)[2] )
   
#
    def test_language_field(self):
        """
        Test for the presence of non-"None" values in a good record,
        and "None"s in an intentionally-damaged record.
        """ 

        # this record contains an valid language key:value pair 
        GOOD_LANG_STRING = """{"id":"tag:gnip.tumblr.com:2012:post/45837266561/post","verb":"post","target":{"link":"http://m2lm-photography.tumblr.com/","displayName":"m2lm-photography","objectType":"blog"},"postedTime":"2013-03-20T15:09:40+00:00","actor":{"id":"tag:gnip.tumblr.com:2012:blog/99030041"},"object":{"summary":"<p>Blé</p><p><a href=\\"http://m2lm-photography.tumblr.com\\">http://m2lm-photography.tumblr.com</a></p>","tags":[{"displayName":"M2LM-photography"},{"displayName":"Blé"}],"id":"tag:gnip.tumblr.com:2012:post/45837266561","tumblrReblogKey":"uBR5KD9F","items":[{"summary":"","image":{"height":75,"width":75,"link":"http://24.media.tumblr.com/e3e1d83c86585d9288f443bc2a4f1461/tumblr_mjyss26sZD1s92ligo1_75sq.jpg"},"objectType":"image","fullImage":{"height":853,"width":1280,"link":"http://25.media.tumblr.com/e3e1d83c86585d9288f443bc2a4f1461/tumblr_mjyss26sZD1s92ligo1_1280.jpg"}}],"objectTypes":["image"],"link":"http://m2lm-photography.tumblr.com/post/45837266561/ble-http-m2lm-photography-tumblr-com","totalItems":1,"tumblrType":"photo","postedTime":"2013-03-20T15:09:38+00:00","tumblrFormat":"html","objectType":"photo-album"},"gnip":{"urls":[{"url":"http://m2lm-photography.tumblr.com","expanded_url":"http://m2lm-photography.tumblr.com"}],"language":{"value":"en"}}}"""

        # this record has a bad language key 
        BAD_LANG_STRING = """{"id":"tag:gnip.tumblr.com:2012:post/45837266561/post","verb":"post","target":{"link":"http://m2lm-photography.tumblr.com/","displayName":"m2lm-photography","objectType":"blog"},"postedTime":"2013-03-20T15:09:40+00:00","actor":{"id":"tag:gnip.tumblr.com:2012:blog/99030041"},"object":{"summary":"<p>Blé</p><p><a href=\\"http://m2lm-photography.tumblr.com\\">http://m2lm-photography.tumblr.com</a></p>","tags":[{"displayName":"M2LM-photography"},{"displayName":"Blé"}],"id":"tag:gnip.tumblr.com:2012:post/45837266561","tumblrReblogKey":"uBR5KD9F","items":[{"summary":"","image":{"height":75,"width":75,"link":"http://24.media.tumblr.com/e3e1d83c86585d9288f443bc2a4f1461/tumblr_mjyss26sZD1s92ligo1_75sq.jpg"},"objectType":"image","fullImage":{"height":853,"width":1280,"link":"http://25.media.tumblr.com/e3e1d83c86585d9288f443bc2a4f1461/tumblr_mjyss26sZD1s92ligo1_1280.jpg"}}],"objectTypes":["image"],"link":"http://m2lm-photography.tumblr.com/post/45837266561/ble-http-m2lm-photography-tumblr-com","totalItems":1,"tumblrType":"photo","postedTime":"2013-03-20T15:09:38+00:00","tumblrFormat":"html","objectType":"photo-album"},"gnip":{"urls":[{"url":"http://m2lm-photography.tumblr.com","expanded_url":"http://m2lm-photography.tumblr.com"}]}}"""

        o = self.objs["lang"]

        # this should have data (an empty string)
        for i, record in o.file_reader( json_string=GOOD_LANG_STRING ):
            record_string = o.procRecord(record)
            self.assertNotEqual( u"None", record_string.split(self.delim)[4] )
        
        # this should not have data in the 2nd and 3rd struct fields (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_LANG_STRING ):
            record_string = o.procRecord(record)
            #print(record_string)
            self.assertEqual( u"None", record_string.split(self.delim)[4] )


    def test_rules_field(self):
        """
        Test for the presence of non-"None" values in a good record,
        and "None"s in an intentionally-damaged record.
        """ 

        # this record contains an valid "gnip:matching_rules" key path, 
        GOOD_RULES_STRING = """{"id":"tag:gnip.tumblr.com:2012:post/45837266698/post","tumblrRebloggedRoot":{"author":{"link":"http://the-personal-quotes.tumblr.com/","displayName":"the personal quotes"},"link":"http://the-personal-quotes.tumblr.com/post/18729698371"},"verb":"post","target":{"link":"http://stardustgrass.tumblr.com/","displayName":"stardustgrass","objectType":"blog"},"postedTime":"2013-03-20T15:09:40+00:00","actor":{"id":"tag:gnip.tumblr.com:2012:blog/76962586"},"object":{"tags":[{"displayName":"the help"},{"displayName":"gif"}],"summary":"","tumblrNoteCount":303949,"objectTypes":["image"],"link":"http://stardustgrass.tumblr.com/post/45837266698","postedTime":"2013-03-20T15:09:40+00:00","tumblrFormat":"html","id":"tag:gnip.tumblr.com:2012:post/45837266698","source":{"link":"http://the-personal-quotes.tumblr.com/post/18729698371","displayName":"the-personal-quotes"},"items":[{"summary":"","image":{"height":52,"width":100,"link":"http://25.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_100.gif"},"objectType":"image","fullImage":{"height":260,"width":500,"link":"http://24.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_500.gif"}}],"tumblrReblogKey":"57dc6dOl","totalItems":1,"tumblrType":"photo","objectType":"photo-album"},"tumblrRebloggedFrom":{"author":{"link":"http://illllest.tumblr.com/","displayName":"Unknown Pleasures"},"link":"http://illllest.tumblr.com/post/45832750088"},"gnip":{"matching_rules":[{"value": "startdustgrass","tag":"test"}]}}"""

# no "matching_rules" key in the "gnip" field in this record; should return "[]"
        BAD_RULES_STRING = """{"id":"tag:gnip.tumblr.com:2012:post/45837266698/post","tumblrRebloggedRoot":{"author":{"link":"http://the-personal-quotes.tumblr.com/","displayName":"the personal quotes"},"link":"http://the-personal-quotes.tumblr.com/post/18729698371"},"verb":"post","target":{"link":"http://stardustgrass.tumblr.com/","displayName":"stardustgrass","objectType":"blog"},"postedTime":"2013-03-20T15:09:40+00:00","actor":{"id":"tag:gnip.tumblr.com:2012:blog/76962586"},"object":{"tags":[{"displayName":"the help"},{"displayName":"gif"}],"summary":"","tumblrNoteCount":303949,"objectTypes":["image"],"link":"http://stardustgrass.tumblr.com/post/45837266698","postedTime":"2013-03-20T15:09:40+00:00","tumblrFormat":"html","id":"tag:gnip.tumblr.com:2012:post/45837266698","source":{"link":"http://the-personal-quotes.tumblr.com/post/18729698371","displayName":"the-personal-quotes"},"items":[{"summary":"","image":{"height":52,"width":100,"link":"http://25.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_100.gif"},"objectType":"image","fullImage":{"height":260,"width":500,"link":"http://24.media.tumblr.com/tumblr_m0dbepbg1B1qfdwsio1_500.gif"}}],"tumblrReblogKey":"57dc6dOl","totalItems":1,"tumblrType":"photo","objectType":"photo-album"},"tumblrRebloggedFrom":{"author":{"link":"http://illllest.tumblr.com/","displayName":"Unknown Pleasures"},"link":"http://illllest.tumblr.com/post/45832750088"},"gnip":{}}"""

        o = self.objs["rules"]

        # this should have data (an empty string)
        for i, record in o.file_reader( json_string=GOOD_RULES_STRING ):
            record_string = o.procRecord(record)
            self.assertNotEqual( "None", record_string.split(self.delim)[4] )
            self.assertNotEqual( "[]", record_string.split(self.delim)[4] )
        
        # this should not have data in the 2nd and 3rd struct fields (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_RULES_STRING ):
            record_string = o.procRecord(record)
            self.assertEqual( "[]", record_string.split(self.delim)[4] )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Josh Montague"
__license__="MIT License"

import sys
import unittest
from StringIO import StringIO
from wordpress_acs import *


# valid activity from source (eg from data/) 
VALID_ACTIVITY = """{"id":"tag:gnip.wordpress.com:2012:blog/4885073/post/1089","verb":"post","target":{"summary":"(by jskr)","link":"http://300slow.wordpress.com/","feed":"http://300slow.wordpress.com/feed/","wpBlogId":4885073,"displayName":"300słow","objectType":"blog"},"postedTime":"2013-05-25T15:43:06.000Z","provider":{"link":"http://im.wordpress.com:8008/posts.json","displayName":"WordPress","objectType":"service"},"object":{"id":"article:wordpress.com:4885073:1089","content":"o już czasu jestem, pod dużym wrażeniem sposobów jakimi kościół próbuje mnie rozbawić.  Co najważniejsze, czyni to skutecznie i wręcz czoła chylę przed ich pomysłowością. Mistrzami są zaś w tak zwanym wykręcaniu kota ogonem. Oto jeden z ostatnich przykładow. Jef Kloch, rzecznik tzw. episkopatu odpowiedział wreszcie na pytania dziennikarzy (milczał jak skała przez niemal dwa miesiące) dotyczące pedofilii w polskim kościele. Co odpowiedział? Że kościół nie zbiera informacji na temat nadużyć księży. Oznacza to, że nie zna (przynajmniej oficjalnie) skali tego problemu. Ale wcale nie to jest najśmieszniejsze. Bo pytany dalej dlaczego to nie zbierają tych informacji, ksiądz Kloch z miną pokerzysty odpowiada, że danych tych nie zbierają ze względu na dobro osób pokrzywdzonych to dopiero jest naprawdę śmieszne, wręcz zwala z nóg. Byłbym bardzo ciekaw przeto, na czym zdaniem pana Klocha to dobro pokrzywdzonych polega. Bo niezmiennie od dłuższego już czasu mam graniczące z pewnością przekonanie, że mówiąc o pokrzywdzonych, pan Kloch ma na myśli wyłącznie biednych, napastowanych seksulanie księży, prawdziwe ofiary pedofilii w kościele.  I co? Ręce wprost same składają się do oklasków.","summary":"Od d&#322;u&#380;szego ju&#380; czasu jestem, pod du&#380;ym wra&#380;eniem sposob&#243;w jakimi ko&#347;ci&#243;&#322; pr&#243;buje mnie rozbawi&#263;. &#160;Co najwa&#380;niejsze, czyni to skutecznie i wr&#281;cz czo&#322;a chyl&#281; przed ich pomys&#322;owo&#347;ci&#261;. Mistrzami s&#261; za&#347; w tak zwanym wykr&#281;caniu kota ogonem. Oto jeden z&#160;ostatnich przyk&#322;adow. Ksi&#261;dz J&#243;zef Kloch, rzecznik tzw. episkopatu odpowiedzia&#322; wreszcie na pytania dziennikarzy (milcza&#322; jak ska&#322;a przez niemal [&#8230;]","link":"http://300slow.wordpress.com/2013/05/25/co-kosciol-wie-a-czego-na-wszelki-wypadek-nie-wie/","postedTime":"2013-05-25T15:42:58Z","wpBlogId":4885073,"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","objectType":"article","updatedTime":"2013-05-25T15:42:58Z","wpPostId":1089},"actor":{"id":"person:wordpress.com:45337318","link":"http://gravatar.com/jerzysikora","wpEmailMd5":"dea3b3f6a8265868871a6e12a01a8cf9","displayName":"jsikora","objectType":"person"},"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","gnip":{"language":{"value":"pl"},"urls":[{"url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg","expanded_url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg"}]}}"""

class TestWordpress(unittest.TestCase):
    """Unit tests of common CSV utility functions"""

    def setUp(self):
        """
        """ 

        self.delim = '|'

        # use a dict to make it easier to refer to the particular object cases
        self.objs = { 
                    "base": WPacsCSV(self.delim, None, False, False, False, False)
                    , "user": WPacsCSV(self.delim, None, True, False, False, False)
                    , "rules": WPacsCSV(self.delim, None, False, True, False, False)
                    , "lang": WPacsCSV(self.delim, None, False, False, True, False)
                    , "struct": WPacsCSV(self.delim, None, False, False, False, True)
                    , "keypath": WPacsCSV(self.delim, "target:displayName", False, False, False, False)
                    , "all": WPacsCSV(self.delim, None, False, False, False, True)
                    }

        # set any instance attributes here, avoid hard-coding in test methods 
        self.lengths = {"base": 4, "user": 5, "rules": 5, "lang": 5, "struct": 8, "keypath":5, "all":8}

    def tearDown(self):
        """
        """
        pass

    
    #
    # helpful to group test methods that are related into sections
    #
    
    def test_sample_data(self):
        """ Check that we can use each test object's procRecord method on each record
        in the newsgator_sample.json example file without raising an exception"""
        # get a temporary obj
        tmp = self.objs["base"]
        # grab the correct data file
        datafile = "./data/wp-com_sample.json"

        # loop over all of the test wordpress processing objects
        for o in self.objs.values():
            # loop over records
            for i, record in o.file_reader(datafile):
                record_string = o.procRecord(record)

    def test_reader(self):
        """Test that our file_reader method is working as expected"""
        # base instance
        o = self.objs["base"]
        # without eg for a loop, use .next()
        g = o.file_reader(json_string = VALID_ACTIVITY)
        self.assertIsInstance(g.next(), tuple)

    def test_lengths(self):
        """Check the number of fields being output."""
        for key in self.objs.keys():
            o = self.objs[key]
            for i, record in o.file_reader(json_string = VALID_ACTIVITY):
                failure_msg = "failed while testing the length of the {} case output".format(key)
                record_string = o.procRecord(record)
                self.assertEqual(len(record_string.split(self.delim)),self.lengths[key],failure_msg)

    def test_user_fields(self):
        """ Test for non-None values in a good record when using the 'user' option,
        and None values in an intentionally damaged record"""

        GOOD_USER_STRING = VALID_ACTIVITY

        USER_STRING_W_NULL = """{"id":"tag:gnip.wordpress.com:2012:blog/4885073/post/1089","verb":"post","target":{"summary":"(by jskr)","link":"http://300slow.wordpress.com/","feed":"http://300slow.wordpress.com/feed/","wpBlogId":4885073,"displayName":"300słow","objectType":"blog"},"postedTime":"2013-05-25T15:43:06.000Z","provider":{"link":"http://im.wordpress.com:8008/posts.json","displayName":"WordPress","objectType":"service"},"object":{"id":"article:wordpress.com:4885073:1089","content":"o już czasu jestem, pod dużym wrażeniem sposobów jakimi kościół próbuje mnie rozbawić.  Co najważniejsze, czyni to skutecznie i wręcz czoła chylę przed ich pomysłowością. Mistrzami są zaś w tak zwanym wykręcaniu kota ogonem. Oto jeden z ostatnich przykładow. Jef Kloch, rzecznik tzw. episkopatu odpowiedział wreszcie na pytania dziennikarzy (milczał jak skała przez niemal dwa miesiące) dotyczące pedofilii w polskim kościele. Co odpowiedział? Że kościół nie zbiera informacji na temat nadużyć księży. Oznacza to, że nie zna (przynajmniej oficjalnie) skali tego problemu. Ale wcale nie to jest najśmieszniejsze. Bo pytany dalej dlaczego to nie zbierają tych informacji, ksiądz Kloch z miną pokerzysty odpowiada, że danych tych nie zbierają ze względu na dobro osób pokrzy wdzonych to dopiero jest naprawdę śmieszne, wręcz zwala z nóg. Byłbym bardzo ciekaw przeto, na czym zdaniem pana Klocha to dobro pokrzywdzonych polega. Bo niezmiennie od dłuższego już czasu mam graniczące z pewnością przekonanie, że mówiąc o pokrzywdzonych, pan Kloch ma na myśli wyłącznie biednych, napastowanychseksulanie księży, prawdziwe ofiary pedofilii w kościele.  I co? Ręce wprost same składają się do oklasków.","summary":"Od d&#322;u&#380;szego ju&#380; czasu jestem, pod du&#380;ym wra&#380;eniem sposob&#243;w jakimi ko&#347;ci&#243;&#322; pr&#243;buje mnie rozbawi&#263;. &#160;Co najwa&#380;niejsze, czyni to skutecznie i wr&#281;cz czo&#322;a chyl&#281; przed ich pomys&#322;owo&#347;ci&#261;. Mistrzami s&#261; za&#347; w tak zwanym wykr&#281;caniu kota ogonem. Oto jeden z&#160;ostatnich przyk&#322;adow. Ksi&#261;dz J&#243;zef Kloch, rzecznik tzw. episkopatu odpowiedzia&#322; wreszcie na pytania dziennikarzy (milcza&#322; jak ska&#322;a przez niemal [&#8230;]","link":"http://300slow.wordpress.com/2013/05/25/co-kosciol-wie-a-czego-na-wszelki-wypadek-nie-wie/","postedTime":"2013-05-25T15:42:58Z","wpBlogId":4885073,"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","objectType":"article","updatedTime":"2013-05-25T15:42:58Z","wpPostId":1089},"actor":{"id":null,"link":"http://gravatar.com/jerzysikora","wpEmailMd5":"dea3b3f6a8265868871a6e12a01a8cf9","displayName":"jsikora","objectType":"person"},"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","gnip":{"language":{"value":"pl"},"urls":[{"url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg","expanded_url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg"}]}}""" 

        USER_STRING_WO_FIELD = """{"id":"tag:gnip.wordpress.com:2012:blog/4885073/post/1089","verb":"post","target":{"summary":"(by jskr)","link":"http://300slow.wordpress.com/","feed":"http://300slow.wordpress.com/feed/","wpBlogId":4885073,"displayName":"300słow","objectType":"blog"},"postedTime":"2013-05-25T15:43:06.000Z","provider":{"link":"http://im.wordpress.com:8008/posts.json","displayName":"WordPress","objectType":"service"},"object":{"id":"article:wordpress.com:4885073:1089","content":"o już czasu jestem, pod dużym wrażeniem sposobów jakimi kościół próbuje mnie rozbawić.  Co najważniejsze, czyni to skutecznie i wręcz czoła chylę przed ich pomysłowością. Mistrzami są zaś w tak zwanym wykręcaniu kota ogonem. Oto jeden z ostatnich przykładow. Jef Kloch, rzecznik tzw. episkopatu odpowiedział wreszcie na pytania dziennikarzy (milczał jak skała przez niemal dwa miesiące) dotyczące pedofilii w polskim kościele. Co odpowiedział? Że kościół nie zbiera informacji na temat nadużyć księży. Oznacza to, że nie zna (przynajmniej oficjalnie) skali tego problemu. Ale wcale nie to jest najśmieszniejsze. Bo pytany dalej dlaczego to nie zbierają tych informacji, ksiądz Kloch z miną pokerzysty odpowiada, że danych tych nie zbierają ze względu na dobro osób pokrzy wdzonych to dopiero jest naprawdę śmieszne, wręcz zwala z nóg. Byłbym bardzo ciekaw przeto, na czym zdaniem pana Klocha to dobro pokrzywdzonych polega. Bo niezmiennie od dłuższego już czasu mam graniczące z pewnością przekonanie, że mówiąc o pokrzywdzonych, pan Kloch ma na myśli wyłącznie biednych, napastowanychseksulanie księży, prawdziwe ofiary pedofilii w kościele.  I co? Ręce wprost same składają się do oklasków.","summary":"Od d&#322;u&#380;szego ju&#380; czasu jestem, pod du&#380;ym wra&#380;eniem sposob&#243;w jakimi ko&#347;ci&#243;&#322; pr&#243;buje mnie rozbawi&#263;. &#160;Co najwa&#380;niejsze, czyni to skutecznie i wr&#281;cz czo&#322;a chyl&#281; przed ich pomys&#322;owo&#347;ci&#261;. Mistrzami s&#261; za&#347; w tak zwanym wykr&#281;caniu kota ogonem. Oto jeden z&#160;ostatnich przyk&#322;adow. Ksi&#261;dz J&#243;zef Kloch, rzecznik tzw. episkopatu odpowiedzia&#322; wreszcie na pytania dziennikarzy (milcza&#322; jak ska&#322;a przez niemal [&#8230;]","link":"http://300slow.wordpress.com/2013/05/25/co-kosciol-wie-a-czego-na-wszelki-wypadek-nie-wie/","postedTime":"2013-05-25T15:42:58Z","wpBlogId":4885073,"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","objectType":"article","updatedTime":"2013-05-25T15:42:58Z","wpPostId":1089},"actor":{"id_DAMAGED":"person:wordpress.com:45337318","link":"http://gravatar.com/jerzysikora","wpEmailMd5":"dea3b3f6a8265868871a6e12a01a8cf9","displayName":"jsikora","objectType":"person"},"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","gnip":{"language":{"value":"pl"},"urls":[{"url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg","expanded_url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg"}]}}"""

        o = self.objs['user']

        # this should have data
        failure_msg = "There should have been data in this field, but there wasn't (or the data that was there was wrong)"
        for i, record in o.file_reader(json_string = GOOD_USER_STRING):
            record_string = o.procRecord(record)
            self.assertNotEqual("None", record_string.split(self.delim)[-1], failure_msg)
            self.assertEqual("person:wordpress.com:45337318" , record_string.split(self.delim)[-1], failure_msg)

        # this should not have data (the field is null)
        failure_msg = "Seems like a JSON 'null' value is not being converted to a Python 'None' value"
        for i, record in o.file_reader(json_string = USER_STRING_W_NULL):
            record_string = o.procRecord(record)
            # payload JSON null => python None => check
            self.assertEqual("None", record_string.split(self.delim)[-1], failure_msg)
            
        # this shouldn't have data either (the field is missing)
        failure_msg = "Seems like missing fields are not being returned as 'None'"
        for i, record in o.file_reader(json_string = USER_STRING_WO_FIELD):
            record_string = o.procRecord(record)
            # payload has a 'None' where the missing string was
            self.assertEqual("None", record_string.split(self.delim)[-1], failure_msg)


    def test_keypath(self):
        """ Test for correct values in the specified keypath"""
        o = self.objs['keypath']
         
        KEYPATH_WO_PATH = """{"id":"tag:gnip.wordpress.com:2012:blog/4885073/post/1089","verb":"post","target":{"summary":"(by jskr)","link":"http://300slow.wordpress.com/","feed":"http://300slow.wordpress.com/feed/","wpBlogId":4885073,"BAD_displayName":"300słow","objectType":"blog"},"postedTime":"2013-05-25T15:43:06.000Z","provider":{"link":"http://im.wordpress.com:8008/posts.json","displayName":"WordPress","objectType":"service"},"object":{"id":"article:wordpress.com:4885073:1089","content":"o już czasu jestem, pod dużym wrażeniem sposobów jakimi kościół próbuje mnie rozbawić.  Co najważniejsze, czyni to skutecznie i wręcz czoła chylę przed ich pomysłowością. Mistrzami są zaś w tak zwanym wykręcaniu kota ogonem. Oto jeden z ostatnich przykładow. Jef Kloch, rzecznik tzw. episkopatu odpowiedział wreszcie na pytania dziennikarzy (milczał jak skała przez niemal dwa miesiące) dotyczące pedofilii w polskim kościele. Co odpowiedział? Że kościół nie zbiera informacji na temat nadużyć księży. Oznacza to, że nie zna (przynajmniej oficjalnie) skali tego problemu. Ale wcale nie to jest najśmieszniejsze. Bo pytany dalej dlaczego to nie zbierają tych informacji, ksiądz Kloch z miną pokerzysty odpowiada, że danych tych nie zbierają ze względu na dobro osób pokrzywdzonych to dopiero jest naprawdę śmieszne, wręcz zwala z nóg. Byłbym bardzo ciekaw przeto, na czym zdaniem pana Klocha to dobro pokrzywdzonych polega. Bo niezmiennie od dłuższego już czasu mam graniczące z pewnością przekonanie, że mówiąc o pokrzywdzonych, pan Kloch ma na myśli wyłącznie biednych, napastowanych seksulanie księży, prawdziwe ofiary pedofilii w kościele.  I co? Ręce wprost same składają się do oklasków.","summary":"Od d&#322;u&#380;szego ju&#380; czasu jestem, pod du&#380;ym wra&#380;eniem sposob&#243;w jakimi ko&#347;ci&#243;&#322; pr&#243;buje mnie rozbawi&#263;. &#160;Co najwa&#380;niejsze, czyni to skutecznie i wr&#281;cz czo&#322;a chyl&#281; przed ich pomys&#322;owo&#347;ci&#261;. Mistrzami s&#261; za&#347; w tak zwanym wykr&#281;caniu kota ogonem. Oto jeden z&#160;ostatnich przyk&#322;adow. Ksi&#261;dz J&#243;zef Kloch, rzecznik tzw. episkopatu odpowiedzia&#322; wreszcie na pytania dziennikarzy (milcza&#322; jak ska&#322;a przez niemal [&#8230;]","link":"http://300slow.wordpress.com/2013/05/25/co-kosciol-wie-a-czego-na-wszelki-wypadek-nie-wie/","postedTime":"2013-05-25T15:42:58Z","wpBlogId":4885073,"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","objectType":"article","updatedTime":"2013-05-25T15:42:58Z","wpPostId":1089},"actor":{"id":"person:wordpress.com:45337318","link":"http://gravatar.com/jerzysikora","wpEmailMd5":"dea3b3f6a8265868871a6e12a01a8cf9","displayName":"jsikora","objectType":"person"},"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","gnip":{"language":{"value":"pl"},"urls":[{"url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg","expanded_url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg"}]}}"""

        KEYPATH_WO_FIELD = """{"id":"tag:gnip.wordpress.com:2012:blog/4885073/post/1089","verb":"post","BAD_target":{"summary":"(by jskr)","link":"http://300slow.wordpress.com/","feed":"http://300slow.wordpress.com/feed/","wpBlogId":4885073,"displayName": "stuff","objectType":"blog"},"postedTime":"2013-05-25T15:43:06.000Z","provider":{"link":"http://im.wordpress.com:8008/posts.json","displayName":"WordPress","objectType":"service"},"object":{"id":"article:wordpress.com:4885073:1089","content":"o już czasu jestem, pod dużym wrażeniem sposobów jakimi kościół próbuje mnie rozbawić.  Co najważniejsze, czyni to skutecznie i wręcz czoła chylę przed ich pomysłowością. Mistrzami są zaś w tak zwanym wykręcaniu kota ogonem. Oto jeden z ostatnich przykładow. Jef Kloch, rzecznik tzw. episkopatu odpowiedział wreszcie na pytania dziennikarzy (milczał jak skała przez niemal dwa miesiące) dotyczące pedofilii w polskim kościele. Co odpowiedział? Że kościół nie zbiera informacji na temat nadużyć księży. Oznacza to, że nie zna (przynajmniej oficjalnie) skali tego problemu. Ale wcale nie to jest najśmieszniejsze. Bo pytany dalej dlaczego to nie zbierają tych informacji, ksiądz Kloch z miną pokerzysty odpowiada, że danych tych nie zbierają ze względu na dobro osób pokrzywdzonych to dopiero jest naprawdę śmieszne, wręcz zwala z nóg. Byłbym bardzo ciekaw przeto, na czym zdaniem pana Klocha to dobro pokrzywdzonych polega. Bo niezmiennie od dłuższego już czasu mam graniczące z pewnością przekonanie, że mówiąc o pokrzywdzonych, pan Kloch ma na myśli wyłącznie biednych, napastowanych seksulanie księży, prawdziwe ofiary pedofilii w kościele.  I co? Ręce wprost same składają się do oklasków.","summary":"Od d&#322;u&#380;szego ju&#380; czasu jestem, pod du&#380;ym wra&#380;eniem sposob&#243;w jakimi ko&#347;ci&#243;&#322; pr&#243;buje mnie rozbawi&#263;. &#160;Co najwa&#380;niejsze, czyni to skutecznie i wr&#281;cz czo&#322;a chyl&#281; przed ich pomys&#322;owo&#347;ci&#261;. Mistrzami s&#261; za&#347; w tak zwanym wykr&#281;caniu kota ogonem. Oto jeden z&#160;ostatnich przyk&#322;adow. Ksi&#261;dz J&#243;zef Kloch, rzecznik tzw. episkopatu odpowiedzia&#322; wreszcie na pytania dziennikarzy (milcza&#322; jak ska&#322;a przez niemal [&#8230;]","link":"http://300slow.wordpress.com/2013/05/25/co-kosciol-wie-a-czego-na-wszelki-wypadek-nie-wie/","postedTime":"2013-05-25T15:42:58Z","wpBlogId":4885073,"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","objectType":"article","updatedTime":"2013-05-25T15:42:58Z","wpPostId":1089},"actor":{"id":"person:wordpress.com:45337318","link":"http://gravatar.com/jerzysikora","wpEmailMd5":"dea3b3f6a8265868871a6e12a01a8cf9","displayName":"jsikora","objectType":"person"},"displayName":"Co kościół wie, a czego na wszelki wypadek nie wie","gnip":{"language":{"value":"pl"},"urls":[{"url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg","expanded_url":"http://300slow.files.wordpress.com/2013/05/avatar_new.jpeg"}]}}"""

        # this should have data
        failure_msg = "Looks like the parser never found the specified kaypath"
        for i, record in o.file_reader(json_string = VALID_ACTIVITY):
            record_string = o.procRecord(record)
            self.assertEqual(u'300słow', record_string.split(self.delim)[-1], failure_msg)

        # this should not have data
        failure_msg = "A missing path is evaluating to something other than 'PATH_EMPTY'"
        for i, record in o.file_reader(json_string = KEYPATH_WO_FIELD):
            record_string = o.procRecord(record)
            self.assertEqual("PATH_EMPTY", record_string.split(self.delim)[-1], failure_msg)

        # this should not have data
        failure_msg = "A missing field is evaluating to something other than 'PATH_EMPTY'"
        for i, record in o.file_reader(json_string = KEYPATH_WO_FIELD):
            record_string = o.procRecord(record)
            self.assertEqual("PATH_EMPTY", record_string.split(self.delim)[-1], failure_msg)

if __name__ == "__main__":
    unittest.main()



from setuptools import setup

GNACS_VERSION_NUMBER = "1.0.1"

if '__main__' == __name__:
    setup(
        name='gnacsupdated',
        version=GNACS_VERSION_NUMBER,
        author='Scott Hendrickson, Josh Montague, Jinsub Hong, Jeff Kolb, Brian Lehman, Fiona Pigott, Nikhil Sulegaon',
        author_email='nikhilsulegaon@gmail.com',
        packages=['acscsv'],
        scripts=['gnacs.py'],
        url='https://github.com/nikhilsu/Gnacs',
        download_url='https://github.com/nikhilsu/Gnacs/tags/%s'%(GNACS_VERSION_NUMBER),
        license='LICENSE.txt',
        description='Gnip normalized activity JSON to csv parser (Twitter, Disqus Comments, and Wordpress Posts and Comments)',
        install_requires=[
                    "ujson >= 1.2",
                        ]
        )



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Josh Montague, Fiona Pigott"
__license__="MIT License"

import sys
import unittest
from StringIO import StringIO
from newsgator_acs import *


# valid activity from source (eg from data/) 
VALID_ACTIVITY = '''{"id":"tag:gnip.newsgator.com:2012:feed/9073659/post/24254991653","postedTime":"2013-10-31T19:55:16+00:00","link":"http://tampa.craigslist.org/hil/syd/4163563239.html","verb":"post","actor":{},"object":{"displayName":"Samsung Series 7 Slate 11.6 64GB Cool New (Hillsborough County) $330","content":"The application works fantastic and you will perform lot of things using it. Look for rates, company trade device with great discount. Purchase Samsung Series 7 Slate 11.6 64GB.The screen is really great has true colors and excellent viewing angles.  [...]"},"ngState":"new","ngFeedScore":1,"ngFeedXmlUrl":"http://tampa.craigslist.org/search/sya?query=ipad&amp;srchType=A&amp;format=rss","ngModifiedDate":"Fri, 01 Nov 2013 15:37:35 GMT","ngFeedId":9073659,"ngPostId":24254991653,"ngScore":0,"original_item":{"ng:feedScore":1,"guid":"http://tampa.craigslist.org/hil/syd/4163563239.html","pubDate":"Thu, 31 Oct 2013 19:55:16 GMT","title":"Samsung Series 7 Slate 11.6 64GB Cool New (Hillsborough County) $330","ng:feedXmlUrl":"http://tampa.craigslist.org/search/sya?query=ipad&amp;srchType=A&amp;format=rss","attributes":{"ng:state":"new"},"ng:modifiedDate":"Fri, 01 Nov 2013 15:37:35 GMT","ng:feedId":9073659,"ng:text":{},"ng:postId":24254991653,"description":"The application works fantastic and you will perform lot of things using it. Look for rates, company trade device with great discount. Purchase Samsung Series 7 Slate 11.6 64GB.The screen is really great has true colors and excellent viewing angles.  [...]","link":"http://tampa.craigslist.org/hil/syd/4163563239.html","ng:score":0},"gnip":{}}'''

class TestNewsgatorAcs(unittest.TestCase):
    """Unit tests of common CSV utility functions"""

    def setUp(self):
        """
        """ 

        self.delim = '|'

        # use a dict to make it easier to refer to the particular object cases
        self.objs = { 
                    "base": NGacsCSV(self.delim, None, False, False) 
                    , "urls": NGacsCSV(self.delim, None, True, False)
                    , "user": NGacsCSV(self.delim, None, False, True)
                    , "all": NGacsCSV(self.delim, None, True, True)
                    , "keypath": NGacsCSV(self.delim,"original_item,attributes,ng:state",False,False)
            }        
        # set any instance attributes here, avoid hard-coding in test methods 
        self.lengths = { "base": 3, "urls": 5, "user": 4, "all": 6, "keypath": 4}

    def tearDown(self):
        """
        """
        pass

    
    #
    # helpful to group test methods that are related into sections
    #
    
    def test_sample_data(self):
        """
        Check that we can use each test object's procRecord method on each record in the 
        newsgator_sample.json example file without raising an exception."""
        # get a temporary object
        tmp = self.objs["base"]

        # grab the correct data file
        datafile = "./data/newsgator_sample.json"
        
        # loop over all of the test newsgator processing objects
        for o in self.objs.values():
            # loop over records in a test file
            for i, record in o.file_reader(datafile):
                # if there's a problem parsing, this should raise an Exception
                record_string = o.procRecord(record)
                #print record_string

    def test_reader(self):
        """Test that our file_reader method is working as expected"""
        # grab the base instance
        o = self.objs["base"]

        # without eg for a loop, use .next()
        g = o.file_reader(json_string = VALID_ACTIVITY)
        self.assertIsInstance(g.next(), tuple)

    def test_lengths(self):
        """ Check the number of fields being output """
        for key in self.objs.keys():
            o = self.objs[key]
            for i, record in o.file_reader(json_string = VALID_ACTIVITY):
                failure_msg = "failed while testing the length of the {} case output".format(key)
                # procRecord returns a delimited string
                record_string = o.procRecord(record)
                self.assertEqual(len(record_string.split(self.delim)), self.lengths[key], failure_msg)
        
    def test_user_fields(self):
        """ Test for non-None values in a good record when using the 'user' option,
        and None values in an intentionally damaged record"""
        GOOD_USER_STRING = """{"id":"tag:gnip.newsgator.com:2012:feed/9072310/post/24254991675","postedTime":"2013-10-31T18:46:09+00:00","link":"magnet:?xt=urn:btih:3CC1E15CD7DDF8480F47391273C60D6E63E6AAD2&dn=NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","verb":"post","actor":{"displayName":"DVD-Uploader"},"object":{"displayName":"NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","content":"","tags":[{"displayName":"Video / Movies DVDR"}],"comments":{"link":"http://thepiratebay.sx/torrent/9127508"}},"ngState":"new","ngFeedScore":10,"ngFeedXmlUrl":"http://rss.thepiratebay.se/202","ngModifiedDate":"Fri, 01 Nov 2013 15:37:37 GMT","ngFeedId":9072310,"ngPostId":24254991675,"ngScore":0,"original_item":{"category":"Video / Movies DVDR","ng:feedScore":10,"author":"DVD-Uploader","comments":"http://thepiratebay.sx/torrent/9127508","guid":"http://thepiratebay.sx/torrent/9127508/","pubDate":"Thu, 31 Oct 2013 18:46:09 GMT","title":"NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","ng:feedXmlUrl":"http://rss.thepiratebay.se/202","attributes":{"ng:state":"new"},"ng:modifiedDate":"Fri, 01 Nov 2013 15:37:37 GMT","ng:feedId":9072310,"ng:text":{},"ng:postId":24254991675,"description":"","link":"magnet:?xt=urn:btih:3CC1E15CD7DDF8480F47391273C60D6E63E6AAD2&dn=NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","ng:score":0},"gnip":{}}""" 
        
        USER_STRING_W_NULL = """{"id":"tag:gnip.newsgator.com:2012:feed/9072310/post/24254991675","postedTime":"2013-10-31T18:46:09+00:00","link":"magnet:?xt=urn:btih:3CC1E15CD7DDF8480F47391273C60D6E63E6AAD2&dn=NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","verb":"post","actor":{"displayName":null},"object":{"displayName":"NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","content":"","tags":[{"displayName":"Video / Movies DVDR"}],"comments":{"link":"http://thepiratebay.sx/torrent/9127508"}},"ngState":"new","ngFeedScore":10,"ngFeedXmlUrl":"http://rss.thepiratebay.se/202","ngModifiedDate":"Fri, 01 Nov 2013 15:37:37 GMT","ngFeedId":9072310,"ngPostId":24254991675,"ngScore":0,"original_item":{"category":"Video / Movies DVDR","ng:feedScore":10,"author":"DVD-Uploader","comments":"http://thepiratebay.sx/torrent/9127508","guid":"http://thepiratebay.sx/torrent/9127508/","pubDate":"Thu, 31 Oct 2013 18:46:09 GMT","title":"NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","ng:feedXmlUrl":"http://rss.thepiratebay.se/202","attributes":{"ng:state":"new"},"ng:modifiedDate":"Fri, 01 Nov 2013 15:37:37 GMT","ng:feedId":9072310,"ng:text":{},"ng:postId":24254991675,"description":"","link":"magnet:?xt=urn:btih:3CC1E15CD7DDF8480F47391273C60D6E63E6AAD2&dn=NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","ng:score":0},"gnip":{}}"""
        
        USER_STRING_WO_FIELD =  """{"id":"tag:gnip.newsgator.com:2012:feed/9072310/post/24254991675","postedTime":"2013-10-31T18:46:09+00:00","link":"magnet:?xt=urn:btih:3CC1E15CD7DDF8480F47391273C60D6E63E6AAD2&dn=NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","verb":"post","actor":{"BAD_displayName": "DVD-Uploader"},"object":{"displayName":"NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","content":"","tags":[{"displayName":"Video / Movies DVDR"}],"comments":{"link":"http://thepiratebay.sx/torrent/9127508"}},"ngState":"new","ngFeedScore":10,"ngFeedXmlUrl":"http://rss.thepiratebay.se/202","ngModifiedDate":"Fri, 01 Nov 2013 15:37:37 GMT","ngFeedId":9072310,"ngPostId":24254991675,"ngScore":0,"original_item":{"category":"Video / Movies DVDR","ng:feedScore":10,"author":"DVD-Uploader","comments":"http://thepiratebay.sx/torrent/9127508","guid":"http://thepiratebay.sx/torrent/9127508/","pubDate":"Thu, 31 Oct 2013 18:46:09 GMT","title":"NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","ng:feedXmlUrl":"http://rss.thepiratebay.se/202","attributes":{"ng:state":"new"},"ng:modifiedDate":"Fri, 01 Nov 2013 15:37:37 GMT","ng:feedId":9072310,"ng:text":{},"ng:postId":24254991675,"description":"","link":"magnet:?xt=urn:btih:3CC1E15CD7DDF8480F47391273C60D6E63E6AAD2&dn=NCIS.S10.MULTiSUBS.PAL.DVDR-Winters","ng:score":0},"gnip":{}}"""
        
        o = self.objs["user"]

        # this should have data
        failure_msg = "There should have been data in this field, but there wasn't (or the data that was there was wrong)"
        for i, record in o.file_reader(json_string = GOOD_USER_STRING):
            record_string = o.procRecord(record)
            self.assertNotEqual("None", record_string.split(self.delim)[-1], failure_msg)
            self.assertEqual("DVD-Uploader" , record_string.split(self.delim)[-1], failure_msg)

        # this should not have data (the field is null)
        failure_msg = "Seems like a JSON 'null' value is not being converted to a Python 'None' value"
        for i, record in o.file_reader(json_string = USER_STRING_W_NULL):
            record_string = o.procRecord(record)
            # payload JSON null => python None => check
            self.assertEqual("None", record_string.split(self.delim)[-1], failure_msg)
            
        # this shouldn't have data either (the field is missing)
        failure_msg = "Seems like missing fields are not being returned as 'None'"
        for i, record in o.file_reader(json_string = USER_STRING_WO_FIELD):
            record_string = o.procRecord(record)
            # payload has a 'None' where the missing string was
            self.assertEqual("None", record_string.split(self.delim)[-1], failure_msg)

    def test_keypath(self):
        """ Test for correct values in the specified keypath"""
        o = self.objs['keypath']

        KEYPATH_WO_PATH = '''{"id":"tag:gnip.newsgator.com:2012:feed/9073659/post/24254991653","postedTime":"2013-10-31T19:55:16+00:00","link":"http://tampa.craigslist.org/hil/syd/4163563239.html","verb":"post","actor":{},"object":{"displayName":"Samsung Series 7 Slate 11.6 64GB Cool New (Hillsborough County) $330","content":"The application works fantastic and you will perform lot of things using it. Look for rates, company trade device with great discount. Purchase Samsung Series 7 Slate 11.6 64GB.The screen is really great has true colors and excellent viewing angles.  [...]"},"ngState":"new","ngFeedScore":1,"ngFeedXmlUrl":"http://tampa.craigslist.org/search/sya?query=ipad&amp;srchType=A&amp;format=rss","ngModifiedDate":"Fri, 01 Nov 2013 15:37:35 GMT","ngFeedId":9073659,"ngPostId":24254991653,"ngScore":0,"original_item":{"ng:feedScore":1,"guid":"http://tampa.craigslist.org/hil/syd/4163563239.html","pubDate":"Thu, 31 Oct 2013 19:55:16GMT","title":"Samsung Series 7 Slate 11.6 64GB Cool New (Hillsborough County) $330","ng:feedXmlUrl":"http://tampa.craigslist.org/search/sya?query=ipad&amp;srchType=A&amp;format=rss","ng:modifiedDate":"Fri, 01 Nov 2013 15:37:35 GMT","ng:feedId":9073659,"ng:text":{},"ng:postId":24254991653,"description":"The application works fantastic and you will perform lot of things using it. Look for rates, company trade device with great discount. Purchase Samsung Series 7 Slate 11.6 64GB.The screen is really great has true colors and excellent viewing angles.  [...]","link":"http://tampa.craigslist.org/hil/syd/4163563239.html","ng:score":0},"gnip":{}}'''

        KEYPATH_WO_FIELD = '''{"id":"tag:gnip.newsgator.com:2012:feed/9073659/post/24254991653","postedTime":"2013-10-31T19:55:16+00:00","link":"http://tampa.craigslist.org/hil/syd/4163563239.html","verb":"post","actor":{},"object":{"displayName":"Samsung Series 7 Slate 11.6 64GB Cool New (Hillsborough County) $330","content":"The application works fantastic and you will perform lot of things using it. Look for rates, company trade device with great discount. Purchase Samsung Series 7 Slate 11.6 64GB.The screen is really great has true colors and excellent viewing angles.  [...]"},"ngState":"new","ngFeedScore":1,"ngFeedXmlUrl":"http://tampa.craigslist.org/search/sya?query=ipad&amp;srchType=A&amp;format=rss","ngModifiedDate":"Fri, 01 Nov 2013 15:37:35 GMT","ngFeedId":9073659,"ngPostId":24254991653,"ngScore":0,"original_item":{"ng:feedScore":1,"guid":"http://tampa.craigslist.org/hil/syd/4163563239.html","pubDate":"Thu, 31 Oct 2013 19:55:16GMT","title":"Samsung Series 7 Slate 11.6 64GB Cool New (Hillsborough County) $330","ng:feedXmlUrl":"http://tampa.craigslist.org/search/sya?query=ipad&amp;srchType=A&amp;format=rss","attributes":{},"ng:modifiedDate":"Fri, 01 Nov 2013 15:37:35 GMT","ng:feedId":9073659,"ng:text":{},"ng:postId":24254991653,"description":"The application works fantastic and you will perform lot of things using it. Look for rates, company trade device with great discount. Purchase Samsung Series 7 Slate 11.6 64GB.The screen is really great has true colors and excellent viewing angles.  [...]","link":"http://tampa.craigslist.org/hil/syd/4163563239.html","ng:score":0},"gnip":{}}'''

        # this should have data
        failure_msg = "Looks like the parser never found the specified kaypath"
        for i, record in o.file_reader(json_string = VALID_ACTIVITY):
            record_string = o.procRecord(record)
            self.assertEqual("new", record_string.split(self.delim)[-1], failure_msg)

        # this should not have data
        failure_msg = "A missing path is evaluating to something other than 'PATH_EMPTY'"
        for i, record in o.file_reader(json_string = KEYPATH_WO_FIELD):
            record_string = o.procRecord(record)
            self.assertEqual("PATH_EMPTY", record_string.split(self.delim)[-1], failure_msg)

        # this should not have data
        failure_msg = "A missing field is evaluating to something other than 'PATH_EMPTY'"
        for i, record in o.file_reader(json_string = KEYPATH_WO_FIELD):
            record_string = o.procRecord(record)
            self.assertEqual("PATH_EMPTY", record_string.split(self.delim)[-1], failure_msg)

if __name__ == "__main__":
    unittest.main()
